library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use IEEE.std_logic_arith.all;

entity forwarding is
	port (  id_instruction : in std_logic_vector(31 downto 0);
		wb_instruction : in std_logic_vector(31 downto 0);
			id_rs_sel : in std_logic_vector(4 downto 0);
			id_rt_sel : in std_logic_vector(4 downto 0);
			ex_instruction : in std_logic_vector(31 downto 0);
			ex_rs_sel : in std_logic_vector(4 downto 0);
			ex_rt_sel : in std_logic_vector(4 downto 0);
			mem_write_reg_sel : in std_logic_vector(4 downto 0);
			wb_write_reg_sel : in std_logic_vector(4 downto 0);
			rs_mux_sel : out std_logic_vector(1 downto 0);----
			rt_mux_sel : out std_logic_vector(1 downto 0);----
			rs_comp_mux_sel: out std_logic;
			rt_comp_mux_sel: out std_logic;
			sw_select : out std_logic;
			mem_reg_write: in std_logic;
			wb_reg_write: in std_logic);
			
end forwarding;

architecture mixed of forwarding is
	
	signal wb_op : std_logic_vector(5 downto 0);
	signal current_op : std_logic_vector(5 downto 0);
	signal rs : std_logic_vector(4 downto 0);
	signal rt : std_logic_vector(4 downto 0);
	signal second_wb : std_logic_vector(4 downto 0);
	signal third_wb : std_logic_vector(4 downto 0);
	signal id_op : std_logic_vector(5 downto 0);
	
begin

current_op <= ex_instruction(31 downto 26);
rs <= ex_rs_sel(4 downto 0);
rt <= ex_rt_sel(4 downto 0);
second_wb <= mem_write_reg_sel(4 downto 0);
third_wb <= wb_write_reg_sel(4 downto 0);
id_op <= id_instruction(31 downto 26);
wb_op <= wb_instruction(31 downto 26);

process(current_op, rs, rt, second_wb, third_wb)
begin
	-- Handles the case when consumer instruction is R format
	-- R format instruction can consume with either rs or rt
	if(current_op = "000000") then -- R-format
		-- Does second wb = rs? If this is true we dont want to check the next case
		if(rs = "00000") then
			rs_mux_sel <= "00";
		elsif((second_wb = rs) and (mem_reg_write = '1') ) then
			rs_mux_sel <= "01";
	    -- Does third wb = rs?
	    elsif((third_wb = rs) and (wb_reg_write = '1') ) then
			rs_mux_sel <= "10";
		else 
			rs_mux_sel <= "00";
		end if;
		-- Does second wb = rt? If this is true we dont want to check the next case
		if(rt = "00000") then
			rt_mux_sel <= "00";
		elsif((second_wb = rt) and (mem_reg_write = '1')  ) then
			rt_mux_sel <= "01";
		-- Does third wb = rt? 
		elsif((third_wb = rt) and (wb_reg_write = '1')  ) then
			rt_mux_sel <= "10";
		else
			rt_mux_sel <= "00";
		end if;
		if (wb_op =  "100011") then
			if ((third_wb = rs) and (wb_reg_write = '1')) then
				rs_mux_sel <= "11";
			else
				rs_mux_sel <= "00";
			end if;
			
			if ((third_wb = rt) and (wb_reg_write = '1')) then
				rt_mux_sel <= "11";
			else
				rt_mux_sel <= "00";
			end if;
		end if;
	-- Handlers for i format instructions 
	-- Process ADDI, LW, SW
	elsif((current_op = "001000") or (current_op = "100011") or (current_op = "101011"))then
			-- Does second wb = rs? If this is true we dont want to check the next case
		if(rs = "00000") then
			rs_mux_sel <= "00";
		elsif((second_wb = rs) and (mem_reg_write = '1')  ) then
			rs_mux_sel <= "01";
	    -- Does third wb = rs?
	    elsif((third_wb = rs) and (wb_reg_write = '1')  ) then
			rs_mux_sel <= "10";
		else 
			rs_mux_sel <= "00";
		end if;
		
		if(current_op = "101011" and second_wb = rt) then
			sw_select <= '1';
		else
			sw_select <= '0';
		end if;
    end if;	
    -- Evaluating for BEQ
	if(id_op = "000100") then
	    if((second_wb = id_rs_sel) and (mem_reg_write = '1')  ) then
			rs_comp_mux_sel <= '1';
		else
			rs_comp_mux_sel <= '0';
		end if;
		if((second_wb = id_rt_sel) and (mem_reg_write = '1')  ) then
			rt_comp_mux_sel <= '1';
		else
			rt_comp_mux_sel <= '0';
	    end if;	
	end if;
end process;

end mixed;
	