library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity hazard_detection is
	port ( 
			CLK : in std_logic;
			id_rs_sel : in std_logic_vector(4 downto 0);
			id_rt_sel : in std_logic_vector(4 downto 0);
			ex_mem_to_reg : in std_logic;
			ex_rt_sel : in std_logic_vector(4 downto 0);
			ex_wb : in std_logic_vector(4 downto 0);
			o_jump: in std_logic;
			o_branch: in std_logic;
			o_equal: in std_logic;
			id_flush: out std_logic;
			ex_flush: out std_logic;
			o_stall: out std_logic);
end hazard_detection;

architecture mixed of hazard_detection is

	signal ex_op : std_logic_vector(5 downto 0);
	signal ex_rt : std_logic_vector(4 downto 0);

begin

ex_rt <= ex_rt_sel(4 downto 0);

process(ex_rt, CLK, ex_mem_to_reg)
begin
if(falling_edge(CLK)) then
	-- If we are jumping clear if/id pipeline register
	if(o_jump = '1') then
		id_flush <= '1';
	
	
	-- If we are branching and the branch is taken clear if/if pipeline register
	elsif((o_equal = '1') and (o_branch = '1')) then
		id_flush <= '1';
	else
		id_flush <= '0';
	end if;
	
	-- LW 100011
	if(ex_mem_to_reg = '1') then
		
		if(((ex_rt = id_rs_sel) or (ex_rt = id_rt_sel)) and ex_rt /= "00000") then
		o_stall <= '1';
		ex_flush <= '1';
		else
		o_stall <= '0';
		ex_flush <= '0';
		end if;
	
	-- ALU producer to BEQ consumer at distance 1
	elsif((o_branch = '1') and ((ex_wb = id_rs_sel) or (ex_wb = id_rt_sel)) and ex_wb /= "00000") then
		ex_flush <= '1';
		o_stall <= '1';
	
	else
		o_stall <= '0';
		ex_flush <= '0';
	end if;

end if;
	
end process;

end mixed;
	
	